#
# UploadReferenceWidget
# Copyright(C), 2007, Enfold Systems, Inc. - ALL RIGHTS RESERVED
#
# This software is licensed under the Terms and Conditions
# contained within the "LICENSE.txt" file that accompanied
# this software.  Any inquiries concerning the scope or
# enforceability of the license should be addressed to:
#
# Enfold Systems, Inc.
# 4617 Montrose Blvd., Suite C215
# Houston, Texas 77006 USA
# p. +1 713.942.2377 | f. +1 832.201.8856
# www.enfoldsystems.com
# info@enfoldsystems.com
#

from Testing import ZopeTestCase
from StringIO import StringIO
from Products.CMFPlone.utils import _createObjectByType

from Products.UploadReferenceWidget.tests import URWTestCase
from Products.UploadReferenceWidget.widget import UploadReferenceWidget

from Products.UploadReferenceWidget.tests.utils import FakeRequest
from Products.UploadReferenceWidget.tests.utils import FakeFileUpload


class UploadReferenceWidgetTests(URWTestCase.URWTestCase):

    def afterSetUp(self):
        #self.app.REQUEST['SESSION'] = {}
        self.doc = _createObjectByType('Document', self.portal, id='doc')
        self.image = _createObjectByType('Image', self.portal, id='image')
        self.video = _createObjectByType('File', self.portal, id='video')
        self.field = self.doc.getField('relatedItems')

    def testSelectExistingContentProcessWidget(self):
        self.field.widget = UploadReferenceWidget()
        form = {
            'relatedItems': [self.image.UID()],
            'relatedItems_option': 'select',
        }
        expected = [self.image.UID()], {}
        result = self.field.widget.process_form(self.doc, self.field, form)
        self.assertEqual(expected, result)

    def testSelectMultipleExistingContentProcessWidget(self):
        self.field.widget = UploadReferenceWidget()
        form = {
            'relatedItems': [self.image.UID(), self.video.UID()],
            'relatedItems_option': 'select',
        }
        expected = [self.image.UID(), self.video.UID()], {}
        result = self.field.widget.process_form(self.doc, self.field, form)
        self.assertEqual(expected, result)

    def testSelectExistingContentCreateReference(self):
        request = FakeRequest()
        self.field.widget = UploadReferenceWidget()
        form = {
            'relatedItems': [self.image.UID()],
            'relatedItems_option': 'select',
        }
        request.form.update(form)
        self.doc.processForm(REQUEST=request, metadata=True)
        expected = [self.image]
        result = self.doc.getRefs('relatesTo')
        self.assertEqual(expected, result)

    def testSelectMultipleExistingContentCreateReferences(self):
        request = FakeRequest()
        self.field.widget = UploadReferenceWidget()
        form = {
            'relatedItems': [self.image.UID(), self.video.UID()],
            'relatedItems_option': 'select',
        }
        request.form.update(form)
        self.doc.processForm(REQUEST=request, metadata=True)
        expected = [self.image, self.video]
        result = self.doc.getRefs('relatesTo')
        self.assertEqual(expected, result)

    def testUploadImageCreateObjectAndReference(self):
        self.setRoles('Manager')
        request = FakeRequest()
        fileupload = FakeFileUpload(filename='enfold.gif', file=self.IMG)
        self.field.widget = UploadReferenceWidget(startup_directory='test')
        self.portal.invokeFactory('Folder', id='test')
        form = {
            'relatedItems_option': 'upload',
            'relatedItems_file': fileupload,
        }
        request.form.update(form)
        self.doc.processForm(REQUEST=request, metadata=True)
        expected = self.portal.test.objectValues('ATImage')
        result = self.doc.getRefs('relatesTo')
        self.assertEqual(expected, result)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0].getContentType(), 'image/gif')

    def testUploadFileCreateObjectAndReference(self):
        self.setRoles('Manager')
        request = FakeRequest()
        fileupload = FakeFileUpload(filename='enfold.pdf', file=self.PDF)
        self.field.widget = UploadReferenceWidget(startup_directory='test')
        self.portal.invokeFactory('Folder', id='test')
        form = {
            'relatedItems_option': 'upload',
            'relatedItems_file': fileupload,
        }
        request.form.update(form)
        self.doc.processForm(REQUEST=request, metadata=True)
        expected = self.portal.test.objectValues('ATFile')
        result = self.doc.getRefs('relatesTo')
        self.assertEqual(expected, result)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0].getContentType(), 'application/pdf')

    def testUploadFileKeepsCurrentReferences(self):
        self.setRoles('Manager')
        request = FakeRequest()
        fileupload = FakeFileUpload(filename='enfold.pdf', file=self.PDF)
        self.field.widget = UploadReferenceWidget(startup_directory='test')
        self.portal.invokeFactory('Folder', id='test')
        form = {
            'relatedItems': [self.image.UID()],
            'relatedItems_option': 'upload',
            'relatedItems_file': fileupload,
        }
        request.form.update(form)
        self.doc.processForm(REQUEST=request, metadata=True)
        expected = [self.image]
        expected.extend(self.portal.test.objectValues())
        result = self.doc.getRefs('relatesTo')
        self.assertEqual(expected, result)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(UploadReferenceWidgetTests))
    return suite
