#
# UploadReferenceWidget
# Copyright(C), 2007, Enfold Systems, Inc. - ALL RIGHTS RESERVED
#
# This software is licensed under the Terms and Conditions
# contained within the "LICENSE.txt" file that accompanied
# this software.  Any inquiries concerning the scope or
# enforceability of the license should be addressed to:
#
# Enfold Systems, Inc.
# 4617 Montrose Blvd., Suite C215
# Houston, Texas 77006 USA
# p. +1 713.942.2377 | f. +1 832.201.8856
# www.enfoldsystems.com
# info@enfoldsystems.com
#

from os.path import basename
from AccessControl import ClassSecurityInfo
from ZPublisher.HTTPRequest import FileUpload
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import _createObjectByType
from Products.Archetypes.Registry import registerWidget
from Products.Archetypes.Registry import registerPropertyType
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import \
        ReferenceBrowserWidget


class UploadReferenceWidget(ReferenceBrowserWidget):

    _properties = ReferenceBrowserWidget._properties.copy()
    _properties.update({
        'macro' : 'uploadreference',
        })

    security = ClassSecurityInfo()

    security.declarePublic('process_form')
    def process_form(self, instance, field, form, empty_marker=None,
                     emptyReturnsMarker=False):
        """A custom implementation for the widget form processing."""

        # As process_form is called half-dozen times, we make it fast
        processed = instance.REQUEST.get('%s_uid' % field.getName())
        if processed:
            return processed, {}

        option = form.get('%s_option' % field.getName(), empty_marker)
        value = form.get(field.getName(), empty_marker)

        # In this case, just do the trivial
        if option == 'select':
            if value is empty_marker:
                return empty_marker
            if emptyReturnsMarker and value == '':
                return empty_marker
            return value, {}

        # In this case, we need to create a new object and get its UID
        if option == 'upload':
            fileobj = form.get('%s_file' % field.getName(), empty_marker)

            # Normalize the filename
            filename = getattr(fileobj, 'filename', '')
            filename = basename(filename)

            if filename:
                portal = getToolByName(instance, 'portal_url').getPortalObject()

                # Guess the mimetype
                mt_tool = getToolByName(portal, 'mimetypes_registry')
                mimetype = str(mt_tool.classify(fileobj.read(1024)))

                # Define the content-type class
                content = 'File'
                if mimetype.startswith('image'):
                    content = 'Image'

                # Define the destination folder
                foldername = self.startup_directory
                if foldername.startswith('/'):
                    foldername = foldername[1:]
                folder = portal.restrictedTraverse(foldername)

                # Create the new thing
                old_id = folder.generateUniqueId(content)
                new_id = folder.invokeFactory(content, id=old_id, title=filename)
                obj = getattr(folder, new_id)
                obj._renameAfterCreation()
                obj.unmarkCreationFlag()
                obj.update_data(fileobj, mimetype)

                # Mark the REQUEST to avoid re-processing
                result = value + [obj.UID()]
                instance.REQUEST.set('%s_uid' % field.getName(), result)

                return result, {}

        return empty_marker


registerWidget(UploadReferenceWidget,
               title='Upload Reference Widget',
               description=('A widget that allows you to upload a content and create a reference to it.'),
               used_for=('Products.Archetypes.Field.ReferenceField',)
               )

#registerPropertyType('default_search_index', 'string', UploadReferenceWidget)
